'
'  RS485.BAS
'
'  Test program for 2-wire RS485.              
'                                              
'  This program assumes that RTS controls the  
'  UART transmitter, as is typical in 2-wire   
'  RS-485 circuits. Note that in a 2-wire      
'  setup, all data is echoed back locally.     
'                                               
'  Execute this program from the DOS command   
'  line (boot to DOS in Windows). 
'  To run from the VB IDE, type:
'
'       VBDOS /LPCL4VB.QLB COMTYPE.BAS
'
'$INCLUDE: 'PCL4VB.BI'

RxBuf512$ = STRING$(512+16,0)   'must be 1st string statement
TxBuf512$ = STRING$(512+16,0)   'must be 2nd string statement

DIM TxBufSeg AS INTEGER
DIM RxBufSeg AS INTEGER
DIM Port AS INTEGER
DIM Code AS INTEGER
DIM I AS INTEGER
DIM L AS INTEGER
DIM LibVer AS INTEGER
DIM BaudCode AS INTEGER
DIM AnyKey AS STRING
DIM Rates(9) AS STRING
DIM UartString(4) AS STRING
DIM PARMS AS STRING
DIM ARG1 AS STRING
DIM ARG2 AS STRING
'init baud rate strings
Rates$(1) = "300"
Rates$(2) = "1200"
Rates$(3) = "2400"
Rates$(4) = "4800"
Rates$(5) = "9600"
Rates$(6) = "19200"
Rates$(7) = "38400"
Rates$(8) = "57600"
Rates$(9) = "115200"

'init UART type strings
UartString(0) = "8250/16450"
UartString(1) = "16550"
UartString(2) = "16650"
UartString(3) = "16750"

'get command line arguments
PARMS$ = RTRIM$(COMMAND$)
I = INSTR(PARMS$, " ")
IF I < 1 THEN
  PRINT "  Usage: RS485 <Port> <BaudRate>"
  PRINT "Example: RS485 1 9600"
  END
END IF
L = LEN(PARMS$)
ARG1$ = LEFT$(PARMS$, I - 1)
ARG2$ = LTRIM$(RIGHT$(PARMS$, L - I))
BaudCode = -1

'Get COM port
Port = VAL(ARG1$) - 1

'Get baud code
FOR I = 1 TO 9
  IF Rates$(I) = ARG2$ THEN
     BaudCode = I
     EXIT FOR
  END IF
NEXT I

'Verify good baud rate
IF BaudCode = -1 THEN
  PRINT "Cannot recognize baud rate "; ARG2$
  STOP
END IF

' set parmameters for the port
Code = SioParms(Port,NoParity,OneStopBit,WordLength8)
'setup RX buffer
RxBufSeg = (1+SSEG(RxBuf512$)) + (SADD(RxBuf512$) \ 16)
Code = SioRxBuf(Port, RxBufSeg, Size512)
IF Code < 0 THEN
  PRINT "Error "; Code; " returned from SioRxBuf()"
  Code = SioError(Code)
  STOP
END IF

IF SioInfo(ASC("I")) <> 0 THEN
  'setup TX buffer
  TxBufSeg = (1+SSEG(TxBuf512$)) + (SADD(TxBuf512$) \ 16)
  Code = SioTxBuf(Port, TxBufSeg, Size512)
  IF Code < 0 THEN
    PRINT "Error "; Code; " returned from SioTxBuf()"
    Code = SioError(Code)
    STOP
  END IF
END IF

'Reset port
Code = SioReset(Port, BaudCode)
IF Code < 0 THEN
  PRINT "Error "; Code; " returned from SioReset()"
  Code = SioError(Code)
  STOP
END IF

CLS
'display info
PRINT "RS485 12/21/97"
PRINT "   Port : COM"; LTRIM$(STR$(1+Port))
PRINT "   Baud : ";Rates$(BaudCode)
LibVer = SioInfo(ASC("V"))
PRINT "Library : "; LTRIM$(STR$(LibVer \ 16)); ".";
PRINT LTRIM$(STR$(LibVer MOD 16))
PRINT "TX Intr : ";
IF SioInfo(ASC("I")) <> 0 THEN
   PRINT "enabled."
ELSE
   PRINT "not enabled."
END IF
' use FIFO if present
Code = SioFIFO(Port,LEVEL_4TH)
PRINT "   UART : "; UartString(Code)

PRINT
PRINT "Entering TERMINAL loop (^Z to quit)..."

DO
  'Anything incoming ?
  Code = SioGetc(Port, 1)
  IF Code = 13 THEN
    PRINT
  ELSEIF Code >= &H20 THEN
    PRINT CHR$(Code);
  END IF
  'Anything to send ?
  AnyKey$ = INKEY$
  IF AnyKey$ <> "" THEN
    'Exit if user types Control-Z
    IF AnyKey$ = CHR$(26) THEN EXIT DO
    ' set RTS 
    Code = SioRTS(Port, SET.LINE)
    ' send character 
    Code = SioPutc(Port,ASC(AnyKey$))
    ' wait till all data has been sent 
    Code = SioEmpty(Port)
    ' clear (drop) RTS 
    Code = SioRTS(Port, CLEAR.LINE)
  END IF
LOOP

Code = SioDone(Port)

END

